Package: acsmoe
Title: Propagate Uncertainty for ACS Tabular Estimates
Version: 0.1.0
Authors@R: 
    person("Dmitry", "Shkolnik", email = "shkolnikd@gmail.com", role = c("aut", "cre"))
Description: Utilities for propagating uncertainty in American Community
    Survey tabular workflows that use published estimates and margins of error,
    following U.S. Census Bureau derived-estimate guidance and complementing
    'tidycensus' margin-of-error workflows. Includes covariance-aware derived
    estimates, simulation helpers, geographic aggregation, confidence-interval
    conversion, and reliability diagnostics.
License: MIT + file LICENSE
URL: https://dshkol.github.io/acsmoe/, https://github.com/dshkol/acsmoe
BugReports: https://github.com/dshkol/acsmoe/issues
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.2
Imports: stats
Suggests: dplyr, ggplot2, knitr, rmarkdown, scales, sf, testthat (>=
        3.0.0), tidycensus
VignetteBuilder: knitr
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-05-26 02:01:42 UTC; dmitryshkolnik
Author: Dmitry Shkolnik [aut, cre]
Maintainer: Dmitry Shkolnik <shkolnikd@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-29 10:40:07 UTC
Built: R 4.6.0; ; 2026-05-29 11:00:58 UTC; unix

# Generated by roxygen2: do not edit by hand

export(acs_aggregate)
export(acs_cv)
export(acs_diff)
export(acs_linear)
export(acs_product)
export(acs_prop)
export(acs_ratio)
export(acs_reliability)
export(acs_simulate)
export(acs_simulate_fn)
export(acs_sum)
export(moe_ci)
export(moe_to_se)
export(se_to_moe)

# acsmoe

`acsmoe` propagates uncertainty in American Community Survey tabular
workflows. It is scoped to estimate + MOE workflows, not ACS data fetching or
microdata variance estimation.

It provides:

- covariance-aware sums, differences, ratios, proportions, products, and linear
  combinations;
- simulation helpers for derived quantities;
- grouped geographic-style aggregation of paired estimate/MOE columns;
- MOE/SE/CI conversion utilities;
- coefficient-of-variation reliability diagnostics.

With no covariance supplied, results match the standard zero-covariance ACS
approximation formulas used by `tidycensus`, so it drops into existing
estimate/MOE pipelines without changing the baseline numbers.

## Installation

```r
# install.packages("acsmoe")  # once on CRAN

# Development version
# install.packages("pak")
pak::pak("dshkol/acsmoe")
```

## References and attribution

The zero-covariance formulas match the U.S. Census Bureau's ACS guidance for
derived estimates, especially Chapter 8 of *Understanding and Using American
Community Survey Data: What All Data Users Need to Know*.

The standard R workflow and baseline MOE helpers are provided by `tidycensus`;
see Walker and Herman's package and Walker's MOE vignette:
<https://walker-data.com/tidycensus/articles/margins-of-error.html>.

The broader motivation for ACS uncertainty handling and regionalization comes
from Spielman, Folch, Nagle, Arribas-Bel, and Koschinsky's ACS uncertainty
papers. The old `geoss/censumander` repository is cited as historical reference
material only; regionalization is not part of this package.

#' Calculate coefficient of variation from an ACS estimate and MOE.
#'
#' @param estimate Numeric estimate.
#' @param moe Numeric MOE.
#' @param conf Confidence level associated with `moe`.
#' @return Numeric coefficient of variation, using standard error divided by
#'   absolute estimate.
#' @examples
#' acs_cv(estimate = 1000, moe = 80)
#' acs_cv(estimate = c(1000, 100, 0), moe = c(80, 60, 5))
#' @export
acs_cv <- function(estimate, moe, conf = 0.90) {
  validate_numeric(estimate, "estimate")
  validate_numeric(moe, "moe")
  args <- recycle_common(estimate, moe, names = c("estimate", "moe"))
  se <- moe_to_se(args$moe, conf = conf)
  cv <- se / abs(args$estimate)
  cv[args$estimate == 0] <- Inf
  cv
}

#' Categorize ACS estimate reliability by CV thresholds.
#'
#' @param estimate Numeric estimate.
#' @param moe Numeric MOE.
#' @param conf Confidence level associated with `moe`.
#' @param thresholds Named numeric vector with `reliable` and `caveat` CV
#'   thresholds. Defaults of 0.12 and 0.40 reflect commonly used applied
#'   conventions (e.g., reliable below 12% CV, unreliable above 40% CV); these
#'   are not a Census Bureau standard and should be adjusted to your domain,
#'   agency guidance, or project-specific quality standard.
#' @return Ordered factor with levels `reliable`, `caveat`, `unreliable`.
#' @examples
#' acs_reliability(estimate = 1000, moe = 80)
#' acs_reliability(estimate = c(1000, 200, 50), moe = c(80, 60, 50))
#' @export
acs_reliability <- function(estimate, moe,
                            conf = 0.90,
                            thresholds = c(reliable = 0.12, caveat = 0.40)) {
  if (is.null(names(thresholds)) ||
      !all(c("reliable", "caveat") %in% names(thresholds))) {
    stop("`thresholds` must include named values `reliable` and `caveat`.",
         call. = FALSE)
  }
  cv <- acs_cv(estimate, moe, conf = conf)
  out <- rep("unreliable", length(cv))
  out[cv < thresholds[["caveat"]]] <- "caveat"
  out[cv < thresholds[["reliable"]]] <- "reliable"
  factor(out, levels = c("reliable", "caveat", "unreliable"))
}

rmvnorm_eigen <- function(n, mean, cov) {
  validate_covariance_matrix(cov)
  eig <- eigen(cov, symmetric = TRUE)
  values <- pmax(eig$values, 0)
  z <- matrix(stats::rnorm(n * length(mean)), nrow = n)
  sweep(z %*% (eig$vectors %*% diag(sqrt(values), nrow = length(values))), 2,
        mean, "+")
}

simulate_draws <- function(estimates, moes, cov = NULL, n_sims = 1000,
                           dist = c("normal", "censored_normal"),
                           conf = 0.90) {
  dist <- match.arg(dist)
  validate_numeric(estimates, "estimates")
  validate_numeric(moes, "moes")
  if (length(estimates) != length(moes)) {
    stop("`estimates` and `moes` must have the same length.", call. = FALSE)
  }
  if (!is.numeric(n_sims) || length(n_sims) != 1L || n_sims < 1) {
    stop("`n_sims` must be a positive integer.", call. = FALSE)
  }
  v <- covariance_from_inputs(moes, cov, conf = conf)
  draws <- rmvnorm_eigen(as.integer(n_sims), estimates, v)
  if (dist == "censored_normal") {
    warning(
      "`censored_normal` replaces draws below zero with zero. This matches the convention used in Napierala & Denton (2017) and the ACS handbook semantics for zero estimates, but it is censoring rather than statistical truncation: it does not preserve the input mean or variance for cells whose estimate sits close to zero relative to its MOE.",
      call. = FALSE
    )
    draws <- pmax(draws, 0)
  }
  draws
}

#' Simulate ACS estimates from published estimates and MOEs.
#'
#' @param estimates Numeric estimates.
#' @param moes Numeric MOEs corresponding to `estimates`.
#' @param cov Optional covariance matrix on the standard-error scale.
#' @param n_sims Number of Monte Carlo simulations.
#' @param dist Distribution assumption: `"normal"` or `"censored_normal"`. The
#'   censored variant replaces below-zero draws with zero, matching the
#'   convention used in Napierala & Denton (2017) for ACS counts.
#' @param conf Confidence level associated with input MOEs.
#' @return A numeric matrix of simulated draws.
#' @examples
#' set.seed(1)
#' acs_simulate(c(x = 100, y = 50), c(10, 5), n_sims = 5)
#' @export
acs_simulate <- function(estimates, moes, cov = NULL, n_sims = 1000,
                         dist = c("normal", "censored_normal"), conf = 0.90) {
  draws <- simulate_draws(estimates, moes, cov, n_sims, dist, conf)
  colnames(draws) <- names(estimates)
  draws
}

#' Simulate a derived ACS statistic.
#'
#' @param estimates Numeric estimates.
#' @param moes Numeric MOEs corresponding to `estimates`.
#' @param fn Function applied to each simulated row.
#' @param cov Optional covariance matrix on the standard-error scale.
#' @param n_sims Number of Monte Carlo simulations.
#' @param dist Distribution assumption: `"normal"` or `"censored_normal"`. The
#'   censored variant replaces below-zero draws with zero, matching the
#'   convention used in Napierala & Denton (2017) for ACS counts.
#' @param conf Confidence level associated with input MOEs.
#' @param summary Summary to return: `"mean"`, `"median"`, or `"ci"`.
#' @param point For `summary = "ci"`, point estimate to report alongside the
#'   percentile interval: `"mean"` (default) or `"median"`.
#' @return A data frame summarizing the simulated derived statistic.
#' @details For `summary = "ci"`, the returned interval is the central
#'   `conf`-level percentile interval of the simulated derived values. The
#'   reported `estimate` is the chosen `point` summary of those values.
#' @examples
#' set.seed(1)
#' acs_simulate_fn(c(100, 50), c(10, 5), fn = sum, n_sims = 500)
#' set.seed(1)
#' acs_simulate_fn(c(100, 50), c(10, 5), fn = sum, n_sims = 500,
#'                 summary = "ci", conf = 0.90)
#' @export
acs_simulate_fn <- function(estimates, moes, fn, cov = NULL, n_sims = 1000,
                            dist = c("normal", "censored_normal"),
                            conf = 0.90,
                            summary = c("mean", "median", "ci"),
                            point = c("mean", "median")) {
  if (!is.function(fn)) {
    stop("`fn` must be a function.", call. = FALSE)
  }
  dist <- match.arg(dist)
  summary <- match.arg(summary)
  point <- match.arg(point)
  draws <- simulate_draws(estimates, moes, cov = cov, n_sims = n_sims,
                          dist = dist, conf = conf)
  values <- apply(draws, 1, fn)
  if (summary == "mean") {
    return(data.frame(estimate = mean(values), se = stats::sd(values)))
  }
  if (summary == "median") {
    return(data.frame(estimate = stats::median(values), se = stats::sd(values)))
  }
  validate_conf(conf)
  alpha <- (1 - conf) / 2
  qs <- stats::quantile(values, probs = c(alpha, 1 - alpha), names = FALSE)
  centre <- if (point == "median") stats::median(values) else mean(values)
  data.frame(estimate = centre, lower = qs[1], upper = qs[2])
}

test_that("simulation is deterministic with a fixed seed", {
  set.seed(42)
  a <- acs_simulate(c(x = 100, y = 50), c(10, 5), n_sims = 5)
  set.seed(42)
  b <- acs_simulate(c(x = 100, y = 50), c(10, 5), n_sims = 5)
  expect_equal(a, b)
})

test_that("simulation function summarizes derived values", {
  set.seed(42)
  out <- acs_simulate_fn(c(100, 50), c(10, 5), fn = sum, n_sims = 200)
  expect_named(out, c("estimate", "se"))
  expect_true(is.finite(out$estimate))
})

test_that("normal simulations recover input moments approximately", {
  set.seed(42)
  estimates <- c(100, 50)
  moes <- c(10, 5)
  draws <- acs_simulate(estimates, moes, n_sims = 50000, dist = "normal")
  expect_equal(colMeans(draws), estimates, tolerance = 0.1)
  expect_equal(apply(draws, 2, stats::sd), moe_to_se(moes), tolerance = 0.1)
})

test_that("simulate_fn forwards distribution and confidence settings", {
  set.seed(42)
  expect_warning(
    out <- acs_simulate_fn(
      estimates = c(-1, 10),
      moes = c(1, 1),
      fn = sum,
      n_sims = 100,
      dist = "censored_normal",
      conf = 0.95
    ),
    "censoring rather than statistical truncation"
  )
  expect_true(is.finite(out$estimate))
})

test_that("censored_normal replaces below-zero draws with exactly zero", {
  set.seed(7)
  draws <- suppressWarnings(acs_simulate(
    estimates = c(0, 100),
    moes = c(20, 20),
    n_sims = 5000,
    dist = "censored_normal"
  ))
  # All draws non-negative
  expect_true(all(draws >= 0))
  # Column 1 (estimate = 0): under censoring, half the draws should pile at
  # exactly zero. (Under statistical truncation that share would be ~0.)
  expect_gt(mean(draws[, 1] == 0), 0.45)
  expect_lt(mean(draws[, 1] == 0), 0.55)
})

validate_covariance_matrix <- function(cov, tol = 1e-10) {
  if (!is.matrix(cov) || !is.numeric(cov)) {
    stop("Covariance input must be a numeric matrix.", call. = FALSE)
  }
  if (nrow(cov) != ncol(cov)) {
    stop("Covariance matrix must be square.", call. = FALSE)
  }
  if (any(!is.finite(cov))) {
    stop("Covariance matrix must contain only finite values.", call. = FALSE)
  }
  if (!isTRUE(all.equal(cov, t(cov), tolerance = tol))) {
    stop("Covariance matrix must be symmetric.", call. = FALSE)
  }
  eig <- eigen(cov, symmetric = TRUE, only.values = TRUE)$values
  if (min(eig) < -tol) {
    stop("Covariance matrix must be positive semidefinite.", call. = FALSE)
  }
  invisible(cov)
}

covariance_from_inputs <- function(moes, cov, conf = 0.90) {
  validate_numeric(moes, "moes")
  se <- moe_to_se(moes, conf = conf)
  n <- length(se)
  base <- diag(se^2, nrow = n)

  if (is.null(cov)) {
    return(base)
  }

  if (length(cov) == 1L && is.numeric(cov)) {
    out <- matrix(cov, nrow = n, ncol = n)
    diag(out) <- se^2
    validate_covariance_matrix(out)
    return(out)
  }

  if (!is.matrix(cov)) {
    stop("`cov` must be NULL, a scalar covariance, or a covariance matrix.",
         call. = FALSE)
  }
  if (!identical(dim(cov), c(n, n))) {
    stop("`cov` matrix dimensions must match the number of estimates.",
         call. = FALSE)
  }
  validate_covariance_matrix(cov)
  cov
}

covariance_from_correlation <- function(moes, rho, conf = 0.90) {
  se <- moe_to_se(moes, conf = conf)
  n <- length(se)

  if (length(rho) == 1L) {
    if (!is.numeric(rho) || is.na(rho) || rho < -1 || rho > 1) {
      stop("Scalar correlation must be between -1 and 1.", call. = FALSE)
    }
    cor <- matrix(rho, nrow = n, ncol = n)
    diag(cor) <- 1
  } else {
    cor <- rho
    if (!is.matrix(cor) || !identical(dim(cor), c(n, n))) {
      stop("Correlation matrix dimensions must match the number of estimates.",
           call. = FALSE)
    }
    if (any(abs(cor) > 1)) {
      stop("Correlations must be between -1 and 1.", call. = FALSE)
    }
  }

  cov <- cor * tcrossprod(se)
  validate_covariance_matrix(cov)
  cov
}

is_zero_cov <- function(cov) {
  is.null(cov) || (length(cov) == 1L && is.numeric(cov) && identical(as.numeric(cov), 0))
}

resolve_col_vector <- function(cols, data, name) {
  if (!is.character(cols)) {
    stop("`", name, "` must be a character vector of column names.",
         call. = FALSE)
  }
  missing <- setdiff(cols, names(data))
  if (length(missing) > 0) {
    stop("Unknown columns in `", name, "`: ", paste(missing, collapse = ", "),
         call. = FALSE)
  }
  cols
}

aggregate_covariance <- function(row_ids, value_col, moes, cov_strategy,
                                 cov_value, conf) {
  if (cov_strategy == "zero") {
    return(NULL)
  }
  if (cov_strategy == "constant") {
    return(covariance_from_correlation(moes, cov_value, conf = conf))
  }
  if (cov_strategy == "supplied") {
    if (!is.list(cov_value) || is.null(cov_value[[value_col]])) {
      stop("For `cov_strategy = 'supplied'`, `cov_value` must be a named list of full covariance matrices.",
           call. = FALSE)
    }
    full <- cov_value[[value_col]]
    if (!is.matrix(full)) {
      stop("Supplied covariance for `", value_col, "` must be a matrix.",
           call. = FALSE)
    }
    return(full[row_ids, row_ids, drop = FALSE])
  }
  stop("Unsupported covariance strategy.", call. = FALSE)
}

#' Aggregate paired ACS estimate and MOE columns by group.
#'
#' @param data A data frame containing ACS estimate and MOE columns.
#' @param group_var Name of the grouping column, supplied as a single string.
#' @param value_cols Character vector of estimate column names to aggregate.
#' @param moe_cols Character vector of MOE column names paired with
#'   `value_cols`.
#' @param cov_strategy Covariance strategy: `"zero"`, `"supplied"`, or
#'   `"constant"`.
#' @param cov_value For `"constant"`, a scalar correlation. For `"supplied"`,
#'   a named list of full covariance matrices keyed by estimate column.
#' @param conf Confidence level associated with input and output MOEs.
#' @details `cov_strategy = "constant"` interprets `cov_value` as a correlation,
#'   not a covariance. This differs from scalar `cov` arguments in core
#'   propagation functions, where a scalar means an off-diagonal covariance on
#'   the standard-error scale.
#'
#'   Output rows are ordered by first appearance of each group level in `data`,
#'   not alphabetically.
#' @return A data frame with one row per group and aggregated estimate/MOE
#'   columns.
#' @examples
#' tracts <- data.frame(
#'   region = c("north", "north", "south", "south"),
#'   pop = c(1000, 1200, 900, 1100),
#'   pop_moe = c(120, 140, 100, 130)
#' )
#' acs_aggregate(tracts, "region", "pop", "pop_moe")
#' acs_aggregate(tracts, "region", "pop", "pop_moe",
#'               cov_strategy = "constant", cov_value = 0.25)
#' @export
acs_aggregate <- function(data, group_var, value_cols, moe_cols,
                          cov_strategy = c("zero", "supplied", "constant"),
                          cov_value = 0, conf = 0.90) {
  cov_strategy <- match.arg(cov_strategy)
  if (!is.character(group_var) || length(group_var) != 1L) {
    stop("`group_var` must be a single column name as a string.",
         call. = FALSE)
  }
  if (!group_var %in% names(data)) {
    stop("`group_var` must identify an existing column: '", group_var,
         "' not found.", call. = FALSE)
  }
  value_cols <- resolve_col_vector(value_cols, data, "value_cols")
  moe_cols <- resolve_col_vector(moe_cols, data, "moe_cols")
  if (length(value_cols) != length(moe_cols)) {
    stop("`value_cols` and `moe_cols` must have the same length.", call. = FALSE)
  }
  groups <- data[[group_var]]
  level_order <- unique(groups)
  level_order <- level_order[!is.na(level_order)]
  split_ids <- split(seq_len(nrow(data)), factor(groups, levels = level_order),
                     drop = TRUE)
  out <- data.frame(stats::setNames(list(names(split_ids)), group_var),
                    stringsAsFactors = FALSE)

  for (i in seq_along(value_cols)) {
    value_col <- value_cols[[i]]
    moe_col <- moe_cols[[i]]
    estimates <- numeric(length(split_ids))
    moes <- numeric(length(split_ids))
    for (g in seq_along(split_ids)) {
      ids <- split_ids[[g]]
      cov <- aggregate_covariance(
        ids, value_col, data[[moe_col]][ids],
        cov_strategy, cov_value, conf
      )
      res <- acs_sum(data[[value_col]][ids], data[[moe_col]][ids],
                     cov = cov, conf = conf)
      estimates[[g]] <- res$estimate
      moes[[g]] <- res$moe
    }
    out[[value_col]] <- estimates
    out[[moe_col]] <- moes
  }

  out
}

test_that("aggregation sums paired estimate and MOE columns", {
  dat <- data.frame(
    region = c("a", "a", "b"),
    total = c(10, 20, 30),
    total_moe = c(2, 3, 4)
  )

  out <- acs_aggregate(dat, "region", "total", "total_moe")
  expect_equal(out$total, c(30, 30))
  expect_equal(out$total_moe[1], acs_sum(c(10, 20), c(2, 3))$moe)
})

test_that("constant aggregation covariance is interpreted as correlation", {
  dat <- data.frame(
    region = c("a", "a"),
    total = c(10, 20),
    total_moe = c(2, 3)
  )
  zero <- acs_aggregate(dat, "region", "total", "total_moe",
                        cov_strategy = "zero")
  corr <- acs_aggregate(dat, "region", "total", "total_moe",
                        cov_strategy = "constant", cov_value = 0.5)
  expect_gt(corr$total_moe, zero$total_moe)
})

acs_z <- function(conf) {
  validate_conf(conf)
  stats::qnorm((1 + conf) / 2)
}

validate_conf <- function(conf) {
  if (!is.numeric(conf) || length(conf) != 1L || is.na(conf) ||
      conf <= 0 || conf >= 1) {
    stop("`conf` must be a single number between 0 and 1.", call. = FALSE)
  }
  invisible(conf)
}

validate_numeric <- function(x, name) {
  if (!is.numeric(x)) {
    stop("`", name, "` must be numeric.", call. = FALSE)
  }
  invisible(x)
}

recycle_common <- function(..., names) {
  args <- list(...)
  lengths <- vapply(args, length, integer(1))
  n <- max(lengths)
  bad <- lengths != 1L & lengths != n
  if (any(bad)) {
    stop("Inputs must have length 1 or a common length.", call. = FALSE)
  }
  args <- Map(rep, args, length.out = n)
  stats::setNames(args, names)
}

#' Convert an ACS margin of error to a standard error.
#'
#' @param moe Numeric margin of error.
#' @param conf Confidence level associated with `moe`.
#' @return Numeric standard error.
#' @examples
#' moe_to_se(c(10, 25, 100))
#' moe_to_se(10, conf = 0.95)
#' @export
moe_to_se <- function(moe, conf = 0.90) {
  validate_numeric(moe, "moe")
  moe / acs_z(conf)
}

#' Convert a standard error to an ACS-style margin of error.
#'
#' @param se Numeric standard error.
#' @param conf Desired confidence level.
#' @return Numeric margin of error.
#' @examples
#' se_to_moe(c(6, 15, 60))
#' se_to_moe(6, conf = 0.95)
#' @export
se_to_moe <- function(se, conf = 0.90) {
  validate_numeric(se, "se")
  se * acs_z(conf)
}

#' Convert an ACS MOE to lower and upper confidence bounds.
#'
#' @param estimate Numeric estimate.
#' @param moe Numeric margin of error.
#' @param conf_in Confidence level associated with `moe`.
#' @param conf_out Desired confidence level for the returned interval.
#' @return A data frame with `estimate`, `lower`, `upper`, and `moe`.
#' @examples
#' moe_ci(estimate = 100, moe = 10)
#' moe_ci(estimate = c(100, 200), moe = c(10, 25), conf_out = 0.99)
#' @export
moe_ci <- function(estimate, moe, conf_in = 0.90, conf_out = 0.95) {
  validate_numeric(estimate, "estimate")
  validate_numeric(moe, "moe")
  args <- recycle_common(estimate, moe, names = c("estimate", "moe"))
  se <- moe_to_se(args$moe, conf = conf_in)
  out_moe <- se_to_moe(se, conf = conf_out)
  data.frame(
    estimate = args$estimate,
    lower = args$estimate - out_moe,
    upper = args$estimate + out_moe,
    moe = out_moe
  )
}

test_that("MOE and SE conversions round-trip", {
  moe <- c(10, 25, 100)
  se <- moe_to_se(moe)
  expect_equal(se_to_moe(se), moe)
})

test_that("confidence intervals widen at higher confidence", {
  ci90 <- moe_ci(100, 10, conf_in = 0.90, conf_out = 0.90)
  ci95 <- moe_ci(100, 10, conf_in = 0.90, conf_out = 0.95)
  expect_lt(ci95$lower, ci90$lower)
  expect_gt(ci95$upper, ci90$upper)
})

test_that("CV reliability thresholds classify estimates", {
  expect_equal(as.character(acs_reliability(1000, 10)), "reliable")
  expect_equal(as.character(acs_reliability(100, 80)), "unreliable")
})

test_that("reliability passes confidence level through to CV calculation", {
  moe95 <- se_to_moe(39, conf = 0.95)
  expect_equal(as.character(acs_reliability(100, moe95, conf = 0.95)), "caveat")
  expect_equal(as.character(acs_reliability(100, moe95, conf = 0.90)), "unreliable")
})

result_df <- function(estimate, moe, se = NULL) {
  out <- data.frame(estimate = estimate, moe = moe)
  if (!is.null(se)) {
    out$se <- se
  }
  out
}

#' Propagate ACS uncertainty for a sum.
#'
#' @param estimates Numeric estimates to sum.
#' @param moes Numeric MOEs corresponding to `estimates`.
#' @param cov Optional covariance matrix on the standard-error scale. A scalar
#'   is interpreted as a common off-diagonal covariance.
#' @param conf Confidence level associated with input and output MOEs.
#' @details A scalar `cov` is a covariance, not a correlation. In contrast,
#'   `acs_aggregate(cov_strategy = "constant")` accepts a scalar correlation
#'   because that interface derives covariances from group-specific MOEs.
#' @return A one-row data frame with `estimate`, `moe`, and `se`.
#' @examples
#' acs_sum(c(100, 50, 25), c(12, 8, 6))
#' acs_sum(c(100, 50), c(12, 8), cov = 5)
#' @export
acs_sum <- function(estimates, moes, cov = NULL, conf = 0.90) {
  validate_numeric(estimates, "estimates")
  validate_numeric(moes, "moes")
  if (length(estimates) != length(moes)) {
    stop("`estimates` and `moes` must have the same length.", call. = FALSE)
  }

  keep <- !is.na(estimates) & !is.na(moes)
  estimates <- estimates[keep]
  moes <- moes[keep]
  if (is_zero_cov(cov)) {
    # Match tidycensus::moe_sum(): multiple zero estimates sharing an MOE
    # contribute that MOE once rather than inflating the derived MOE.
    zeros <- estimates == 0
    moes_for_calc <- c(unique(moes[zeros]), moes[!zeros])
    se <- sqrt(sum(moe_to_se(moes_for_calc, conf = conf)^2))
    return(result_df(sum(estimates), se_to_moe(se, conf = conf), se))
  }
  v <- covariance_from_inputs(moes, cov, conf = conf)
  w <- rep(1, length(estimates))
  se <- sqrt(drop(t(w) %*% v %*% w))
  result_df(sum(estimates), se_to_moe(se, conf = conf), se)
}

#' Propagate ACS uncertainty for a difference.
#'
#' @param estimate1 First estimate.
#' @param moe1 MOE for `estimate1`.
#' @param estimate2 Second estimate.
#' @param moe2 MOE for `estimate2`.
#' @param cov Covariance between the two estimates on the standard-error scale.
#' @param conf Confidence level associated with input and output MOEs.
#' @return A data frame with `estimate`, `moe`, and `se`.
#' @examples
#' acs_diff(100, 10, 40, 8)
#' acs_diff(100, 10, 40, 8, cov = 5)
#' @export
acs_diff <- function(estimate1, moe1, estimate2, moe2, cov = 0, conf = 0.90) {
  args <- recycle_common(
    estimate1, moe1, estimate2, moe2, cov,
    names = c("estimate1", "moe1", "estimate2", "moe2", "cov")
  )
  se1 <- moe_to_se(args$moe1, conf = conf)
  se2 <- moe_to_se(args$moe2, conf = conf)
  var <- se1^2 + se2^2 - 2 * args$cov
  if (any(var < -1e-10, na.rm = TRUE)) {
    stop("Covariance implies a negative variance.", call. = FALSE)
  }
  se <- sqrt(pmax(var, 0))
  result_df(args$estimate1 - args$estimate2, se_to_moe(se, conf = conf), se)
}

#' Propagate ACS uncertainty for a ratio.
#'
#' @param num Numerator estimate.
#' @param num_moe Numerator MOE.
#' @param denom Denominator estimate.
#' @param denom_moe Denominator MOE.
#' @param cov Numerator-denominator covariance on the standard-error scale.
#' @param conf Confidence level associated with input and output MOEs.
#' @return A data frame with `estimate`, `moe`, and `se`.
#' @examples
#' acs_ratio(num = 10, num_moe = 2, denom = 50, denom_moe = 5)
#' @export
acs_ratio <- function(num, num_moe, denom, denom_moe, cov = 0, conf = 0.90) {
  args <- recycle_common(
    num, num_moe, denom, denom_moe, cov,
    names = c("num", "num_moe", "denom", "denom_moe", "cov")
  )
  se_num <- moe_to_se(args$num_moe, conf = conf)
  se_den <- moe_to_se(args$denom_moe, conf = conf)
  estimate <- args$num / args$denom
  var <- (se_num^2 / args$denom^2) +
    ((args$num^2 * se_den^2) / args$denom^4) -
    ((2 * args$num * args$cov) / args$denom^3)
  if (any(var < -1e-10, na.rm = TRUE)) {
    stop("Covariance implies a negative variance.", call. = FALSE)
  }
  se <- sqrt(pmax(var, 0))
  result_df(estimate, se_to_moe(se, conf = conf), se)
}

#' Propagate ACS uncertainty for a proportion.
#'
#' @param num Numerator estimate.
#' @param num_moe Numerator MOE.
#' @param denom Denominator estimate.
#' @param denom_moe Denominator MOE.
#' @param cov Numerator-denominator covariance on the standard-error scale.
#' @param conf Confidence level associated with input and output MOEs.
#' @details This function follows the Census approximation used for proportions
#'   where the numerator is a subset of the denominator. It does not validate
#'   that `num <= denom`; behavior for non-nested ratios is formulaic rather
#'   than a claim that the inputs define a valid proportion.
#' @return A data frame with `estimate`, `moe`, and `se`.
#' @examples
#' acs_prop(num = 10, num_moe = 2, denom = 50, denom_moe = 5)
#' @export
acs_prop <- function(num, num_moe, denom, denom_moe, cov = 0, conf = 0.90) {
  if (is_zero_cov(cov)) {
    args <- recycle_common(
      num, num_moe, denom, denom_moe,
      names = c("num", "num_moe", "denom", "denom_moe")
    )
    estimate <- args$num / args$denom
    ratio_moe <- acs_ratio(
      args$num, args$num_moe, args$denom, args$denom_moe,
      cov = 0, conf = conf
    )$moe
    x <- args$num_moe^2 - (estimate^2 * args$denom_moe^2)
    moe <- ratio_moe
    pos <- x > 0 & !is.na(x)
    moe[pos] <- sqrt(x[pos]) / args$denom[pos]
    return(result_df(estimate, moe, moe_to_se(moe, conf = conf)))
  }
  acs_ratio(num, num_moe, denom, denom_moe, cov = cov, conf = conf)
}

#' Propagate ACS uncertainty for a product.
#'
#' @param estimate1 First estimate.
#' @param moe1 MOE for `estimate1`.
#' @param estimate2 Second estimate.
#' @param moe2 MOE for `estimate2`.
#' @param cov Covariance between the two estimates on the standard-error scale.
#' @param conf Confidence level associated with input and output MOEs.
#' @return A data frame with `estimate`, `moe`, and `se`.
#' @examples
#' acs_product(10, 2, 50, 5)
#' @export
acs_product <- function(estimate1, moe1, estimate2, moe2, cov = 0, conf = 0.90) {
  args <- recycle_common(
    estimate1, moe1, estimate2, moe2, cov,
    names = c("estimate1", "moe1", "estimate2", "moe2", "cov")
  )
  se1 <- moe_to_se(args$moe1, conf = conf)
  se2 <- moe_to_se(args$moe2, conf = conf)
  estimate <- args$estimate1 * args$estimate2
  var <- (args$estimate2^2 * se1^2) +
    (args$estimate1^2 * se2^2) +
    (2 * args$estimate1 * args$estimate2 * args$cov)
  if (any(var < -1e-10, na.rm = TRUE)) {
    stop("Covariance implies a negative variance.", call. = FALSE)
  }
  se <- sqrt(pmax(var, 0))
  result_df(estimate, se_to_moe(se, conf = conf), se)
}

#' Propagate ACS uncertainty for a linear combination.
#'
#' @param estimates Numeric estimates.
#' @param moes Numeric MOEs corresponding to `estimates`.
#' @param weights Numeric weights for the linear combination.
#' @param cov Optional covariance matrix on the standard-error scale. A scalar
#'   is interpreted as a common off-diagonal covariance.
#' @param conf Confidence level associated with input and output MOEs.
#' @details A scalar `cov` is a covariance, not a correlation. In contrast,
#'   `acs_aggregate(cov_strategy = "constant")` accepts a scalar correlation
#'   because that interface derives covariances from group-specific MOEs.
#' @return A one-row data frame with `estimate`, `moe`, and `se`.
#' @examples
#' acs_linear(c(100, 50, 25), c(10, 8, 6), weights = c(2, -1, 0.5))
#' @export
acs_linear <- function(estimates, moes, weights, cov = NULL, conf = 0.90) {
  validate_numeric(estimates, "estimates")
  validate_numeric(moes, "moes")
  validate_numeric(weights, "weights")
  if (length(estimates) != length(moes) || length(estimates) != length(weights)) {
    stop("`estimates`, `moes`, and `weights` must have the same length.",
         call. = FALSE)
  }
  v <- covariance_from_inputs(moes, cov, conf = conf)
  se <- sqrt(drop(t(weights) %*% v %*% weights))
  result_df(sum(weights * estimates), se_to_moe(se, conf = conf), se)
}

test_that("zero-covariance formulas reduce to tidycensus", {
  testthat::skip_if_not_installed("tidycensus")

  estimates <- c(10, 20, 0, 0)
  moes <- c(2, 3, 4, 4)
  expect_equal(acs_sum(estimates, moes)$moe,
               tidycensus::moe_sum(moes, estimates))

  expect_equal(acs_ratio(10, 2, 50, 5)$moe,
               tidycensus::moe_ratio(10, 50, 2, 5))
  expect_equal(acs_prop(10, 2, 50, 5)$moe,
               tidycensus::moe_prop(10, 50, 2, 5))
  expect_equal(acs_product(10, 2, 50, 5)$moe,
               tidycensus::moe_product(10, 50, 2, 5))
})

test_that("proportion formula uses Census fallback when subtraction term is negative", {
  res <- acs_prop(num = 90, num_moe = 5, denom = 100, denom_moe = 20)
  fallback <- acs_ratio(num = 90, num_moe = 5, denom = 100, denom_moe = 20)
  expect_equal(res$moe, fallback$moe)
})

test_that("covariance affects differences with the correct sign", {
  no_cov <- acs_diff(100, 10, 40, 8, cov = 0)
  pos_cov <- acs_diff(100, 10, 40, 8, cov = 10)
  expect_lt(pos_cov$se, no_cov$se)
})

test_that("linear combinations use matrix covariance", {
  moes <- c(10, 20)
  se <- moe_to_se(moes)
  cov <- matrix(c(se[1]^2, 5, 5, se[2]^2), 2)
  res <- acs_linear(c(100, 50), moes, weights = c(1, -1), cov = cov)
  expect_equal(res$estimate, 50)
  expect_equal(res$se, sqrt(se[1]^2 + se[2]^2 - 10))
})

test_that("linear combinations match hand-computed zero-covariance result", {
  estimates <- c(100, 50, 25)
  moes <- c(10, 8, 6)
  weights <- c(2, -1, 0.5)
  res <- acs_linear(estimates, moes, weights)
  expected_se <- sqrt(sum((weights * moe_to_se(moes))^2))
  expect_equal(res$estimate, sum(weights * estimates))
  expect_equal(res$se, expected_se)
})

test_that("invalid covariance matrices are rejected", {
  bad <- matrix(c(1, 2, 2, 1), 2)
  expect_error(acs_linear(c(1, 1), c(1, 1), c(1, 1), cov = bad),
               "positive semidefinite")
})

#' acsmoe: Propagate Uncertainty for ACS Tabular Estimates
#'
#' `acsmoe` provides utilities for American Community Survey workflows that
#' already have published estimates and margins of error. It complements
#' `tidycensus` by matching the standard zero-covariance ACS MOE formulas by
#' default while allowing users to supply covariance structures when available.
#'
#' The package does not download ACS data, estimate variance from microdata, or
#' implement regionalization. Use `tidycensus` for data access and `survey` or
#' `srvyr` for microdata/replicate-weight workflows.
#'
#' Core references include the U.S. Census Bureau ACS handbook guidance on
#' derived estimates, Walker and Herman's `tidycensus` package, and the ACS
#' uncertainty literature by Spielman, Folch, Nagle, Arribas-Bel, and
#' Koschinsky. See the README and vignettes for full references.
#'
#' @section Output shape:
#' Propagation helpers (`acs_sum`, `acs_diff`, `acs_ratio`, `acs_prop`,
#' `acs_product`, `acs_linear`) return a data frame with columns `estimate`,
#' `moe`, and `se`. `moe_ci()` returns `estimate`, `lower`, `upper`, `moe`.
#' `acs_simulate()` returns a numeric matrix of draws. `acs_simulate_fn()`
#' returns either `(estimate, se)` for `summary = "mean"`/`"median"`, or
#' `(estimate, lower, upper)` for `summary = "ci"`. `acs_cv()` and
#' `acs_reliability()` return atomic vectors.
#'
#' @keywords internal
"_PACKAGE"

---
title: "Using acsmoe with tidycensus"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Using acsmoe with tidycensus}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

`tidycensus` is the right tool for downloading ACS estimates and margins of
error. `acsmoe` starts after that: it works with estimate/MOE columns that you
already have.

Kyle Walker's `tidycensus` article on ACS margins of error shows the standard
workflow and the standard Census approximation formulas:
<https://walker-data.com/tidycensus/articles/margins-of-error.html>.

That article demonstrates `tidycensus::moe_sum()`, `tidycensus::moe_prop()`,
`tidycensus::moe_ratio()`, and `tidycensus::moe_product()`. It also quotes the
Census warning that these approximation methods do not account for correlation
or covariance between basic estimates. `acsmoe` is intended for the same
tabular ACS regime, but exposes covariance-aware extensions and a grouped
aggregation helper.

## Pull data with tidycensus

This example mirrors the shape of Walker's MOE vignette: pull tract-level ACS
age-by-sex cells, then aggregate those cells into a derived total. The call is
not evaluated in this vignette because it requires network access and, in many
setups, a Census API key.

```{r, eval = FALSE}
library(tidycensus)
library(dplyr)
library(acsmoe)

vars <- paste0("B01001_0", c(20:25, 44:49))

ramsey <- get_acs(
  geography = "tract",
  variables = vars,
  state = "MN",
  county = "Ramsey",
  year = 2016
)

ramsey65 <- ramsey |>
  group_by(GEOID) |>
  summarize(
    estimate_65plus = sum(estimate),
    moe_65plus = acs_sum(estimate, moe)$moe,
    .groups = "drop"
  )
```

With no covariance supplied, `acs_sum()` intentionally reduces to the same
zero-covariance root-sum-square calculation used by `tidycensus::moe_sum()`.
That makes it a drop-in bridge from the standard workflow to more explicit
uncertainty propagation.

The package website includes a fuller `tidycensus` example with evaluated maps
when the site is built with Census API credentials. That example is kept out of
CRAN vignette evaluation because it requires network access, `sf` geometries,
and current ACS API availability.

## Work from paired estimate/MOE columns

Many ACS workflows become wide after `tidycensus::get_acs(output = "wide")`, or
after a user-created join. `acs_aggregate()` handles this paired-column form.

```{r}
library(acsmoe)

tracts <- data.frame(
  region = c("north", "north", "south", "south"),
  population = c(1000, 1200, 900, 1100),
  population_moe = c(120, 140, 100, 130),
  households = c(420, 500, 360, 440),
  households_moe = c(60, 70, 50, 65)
)

acs_aggregate(
  tracts,
  group_var = "region",
  value_cols = c("population", "households"),
  moe_cols = c("population_moe", "households_moe")
)
```

The default `cov_strategy = "zero"` is deliberately conservative in the API
sense: it matches the standard Census approximation behavior. It should not be
read as a claim that tract estimates are truly independent.

## Add covariance when you have it

If a covariance matrix is available from an external method, pass it on the
standard-error scale. Do not pass covariance of MOEs.

```{r}
estimates <- c(1000, 1200)
moes <- c(120, 140)
ses <- moe_to_se(moes)

cov_mat <- matrix(
  c(ses[1]^2, 1500,
    1500, ses[2]^2),
  nrow = 2
)

acs_sum(estimates, moes, cov = cov_mat)
```

For aggregation, `cov_strategy = "constant"` accepts a scalar correlation and
constructs a valid covariance matrix from the input MOEs. This is useful for
sensitivity analysis, not as an automatic estimator of ACS covariance.

```{r}
acs_aggregate(
  tracts,
  group_var = "region",
  value_cols = "population",
  moe_cols = "population_moe",
  cov_strategy = "constant",
  cov_value = 0.25
)
```

## What this package does not do

`acsmoe` does not download ACS data. Use `tidycensus` for that.

`acsmoe` does not estimate variance from microdata. Use `survey` or `srvyr` for
PUMS and replicate-weight workflows.

`acsmoe` also does not implement regionalization. Walker's `tidycensus` MOE
article points readers to Spielman and Folch's regionalization work and an old
Python implementation. That historical code lives at
<https://github.com/geoss/censumander>. We used it as development-only
reference material for formula checks, but regionalization itself is out of
scope for this package.

The boundary is intentional: `acsmoe` focuses on propagation of uncertainty for
tabular estimate/MOE workflows after ACS data have already been obtained.

## References

- U.S. Census Bureau. 2020. *Understanding and Using American Community Survey
  Data: What All Data Users Need to Know*. See Chapter 8, "Calculating Measures
  of Error for Derived Estimates."
  <https://www.census.gov/programs-surveys/acs/library/handbooks/general.html>
- Walker, Kyle, and Matt Herman. 2025. `tidycensus`: Load US Census Boundary
  and Attribute Data as `tidyverse` and `sf`-Ready Data Frames.
  <https://CRAN.R-project.org/package=tidycensus>
- Walker, Kyle. "Margins of error in the ACS."
  <https://walker-data.com/tidycensus/articles/margins-of-error.html>
- Spielman, Seth E., David Folch, and Nicholas Nagle. 2014. "Patterns and
  Causes of Uncertainty in the American Community Survey." *Applied Geography*
  46: 147-157. <https://doi.org/10.1016/j.apgeog.2013.11.002>
- Spielman, Seth E., and David C. Folch. 2015. "Reducing Uncertainty in the
  American Community Survey through Data-Driven Regionalization." *PLOS ONE*
  10(2): e0115626. <https://doi.org/10.1371/journal.pone.0115626>
- Folch, David C., Daniel Arribas-Bel, Julia Koschinsky, and Seth E. Spielman.
  2016. "Spatial Variation in the Quality of American Community Survey
  Estimates." *Demography* 53(5): 1535-1554.
  <https://link.springer.com/article/10.1007/s13524-016-0499-1>
- Folch, David C., and Seth E. Spielman. `geoss/censumander`. Historical Python
  reference implementation used here only for development validation.
  <https://github.com/geoss/censumander>

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conversions.R
\name{moe_to_se}
\alias{moe_to_se}
\title{Convert an ACS margin of error to a standard error.}
\usage{
moe_to_se(moe, conf = 0.9)
}
\arguments{
\item{moe}{Numeric margin of error.}

\item{conf}{Confidence level associated with \code{moe}.}
}
\value{
Numeric standard error.
}
\description{
Convert an ACS margin of error to a standard error.
}
\examples{
moe_to_se(c(10, 25, 100))
moe_to_se(10, conf = 0.95)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core.R
\name{acs_prop}
\alias{acs_prop}
\title{Propagate ACS uncertainty for a proportion.}
\usage{
acs_prop(num, num_moe, denom, denom_moe, cov = 0, conf = 0.9)
}
\arguments{
\item{num}{Numerator estimate.}

\item{num_moe}{Numerator MOE.}

\item{denom}{Denominator estimate.}

\item{denom_moe}{Denominator MOE.}

\item{cov}{Numerator-denominator covariance on the standard-error scale.}

\item{conf}{Confidence level associated with input and output MOEs.}
}
\value{
A data frame with \code{estimate}, \code{moe}, and \code{se}.
}
\description{
Propagate ACS uncertainty for a proportion.
}
\details{
This function follows the Census approximation used for proportions
where the numerator is a subset of the denominator. It does not validate
that \code{num <= denom}; behavior for non-nested ratios is formulaic rather
than a claim that the inputs define a valid proportion.
}
\examples{
acs_prop(num = 10, num_moe = 2, denom = 50, denom_moe = 5)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate.R
\name{acs_simulate}
\alias{acs_simulate}
\title{Simulate ACS estimates from published estimates and MOEs.}
\usage{
acs_simulate(
  estimates,
  moes,
  cov = NULL,
  n_sims = 1000,
  dist = c("normal", "censored_normal"),
  conf = 0.9
)
}
\arguments{
\item{estimates}{Numeric estimates.}

\item{moes}{Numeric MOEs corresponding to \code{estimates}.}

\item{cov}{Optional covariance matrix on the standard-error scale.}

\item{n_sims}{Number of Monte Carlo simulations.}

\item{dist}{Distribution assumption: \code{"normal"} or \code{"censored_normal"}. The
censored variant replaces below-zero draws with zero, matching the
convention used in Napierala & Denton (2017) for ACS counts.}

\item{conf}{Confidence level associated with input MOEs.}
}
\value{
A numeric matrix of simulated draws.
}
\description{
Simulate ACS estimates from published estimates and MOEs.
}
\examples{
set.seed(1)
acs_simulate(c(x = 100, y = 50), c(10, 5), n_sims = 5)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate.R
\name{acs_simulate_fn}
\alias{acs_simulate_fn}
\title{Simulate a derived ACS statistic.}
\usage{
acs_simulate_fn(
  estimates,
  moes,
  fn,
  cov = NULL,
  n_sims = 1000,
  dist = c("normal", "censored_normal"),
  conf = 0.9,
  summary = c("mean", "median", "ci"),
  point = c("mean", "median")
)
}
\arguments{
\item{estimates}{Numeric estimates.}

\item{moes}{Numeric MOEs corresponding to \code{estimates}.}

\item{fn}{Function applied to each simulated row.}

\item{cov}{Optional covariance matrix on the standard-error scale.}

\item{n_sims}{Number of Monte Carlo simulations.}

\item{dist}{Distribution assumption: \code{"normal"} or \code{"censored_normal"}. The
censored variant replaces below-zero draws with zero, matching the
convention used in Napierala & Denton (2017) for ACS counts.}

\item{conf}{Confidence level associated with input MOEs.}

\item{summary}{Summary to return: \code{"mean"}, \code{"median"}, or \code{"ci"}.}

\item{point}{For \code{summary = "ci"}, point estimate to report alongside the
percentile interval: \code{"mean"} (default) or \code{"median"}.}
}
\value{
A data frame summarizing the simulated derived statistic.
}
\description{
Simulate a derived ACS statistic.
}
\details{
For \code{summary = "ci"}, the returned interval is the central
\code{conf}-level percentile interval of the simulated derived values. The
reported \code{estimate} is the chosen \code{point} summary of those values.
}
\examples{
set.seed(1)
acs_simulate_fn(c(100, 50), c(10, 5), fn = sum, n_sims = 500)
set.seed(1)
acs_simulate_fn(c(100, 50), c(10, 5), fn = sum, n_sims = 500,
                summary = "ci", conf = 0.90)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core.R
\name{acs_linear}
\alias{acs_linear}
\title{Propagate ACS uncertainty for a linear combination.}
\usage{
acs_linear(estimates, moes, weights, cov = NULL, conf = 0.9)
}
\arguments{
\item{estimates}{Numeric estimates.}

\item{moes}{Numeric MOEs corresponding to \code{estimates}.}

\item{weights}{Numeric weights for the linear combination.}

\item{cov}{Optional covariance matrix on the standard-error scale. A scalar
is interpreted as a common off-diagonal covariance.}

\item{conf}{Confidence level associated with input and output MOEs.}
}
\value{
A one-row data frame with \code{estimate}, \code{moe}, and \code{se}.
}
\description{
Propagate ACS uncertainty for a linear combination.
}
\details{
A scalar \code{cov} is a covariance, not a correlation. In contrast,
\code{acs_aggregate(cov_strategy = "constant")} accepts a scalar correlation
because that interface derives covariances from group-specific MOEs.
}
\examples{
acs_linear(c(100, 50, 25), c(10, 8, 6), weights = c(2, -1, 0.5))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core.R
\name{acs_ratio}
\alias{acs_ratio}
\title{Propagate ACS uncertainty for a ratio.}
\usage{
acs_ratio(num, num_moe, denom, denom_moe, cov = 0, conf = 0.9)
}
\arguments{
\item{num}{Numerator estimate.}

\item{num_moe}{Numerator MOE.}

\item{denom}{Denominator estimate.}

\item{denom_moe}{Denominator MOE.}

\item{cov}{Numerator-denominator covariance on the standard-error scale.}

\item{conf}{Confidence level associated with input and output MOEs.}
}
\value{
A data frame with \code{estimate}, \code{moe}, and \code{se}.
}
\description{
Propagate ACS uncertainty for a ratio.
}
\examples{
acs_ratio(num = 10, num_moe = 2, denom = 50, denom_moe = 5)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acsmoe-package.R
\docType{package}
\name{acsmoe-package}
\alias{acsmoe}
\alias{acsmoe-package}
\title{acsmoe: Propagate Uncertainty for ACS Tabular Estimates}
\description{
\code{acsmoe} provides utilities for American Community Survey workflows that
already have published estimates and margins of error. It complements
\code{tidycensus} by matching the standard zero-covariance ACS MOE formulas by
default while allowing users to supply covariance structures when available.
}
\details{
The package does not download ACS data, estimate variance from microdata, or
implement regionalization. Use \code{tidycensus} for data access and \code{survey} or
\code{srvyr} for microdata/replicate-weight workflows.

Core references include the U.S. Census Bureau ACS handbook guidance on
derived estimates, Walker and Herman's \code{tidycensus} package, and the ACS
uncertainty literature by Spielman, Folch, Nagle, Arribas-Bel, and
Koschinsky. See the README and vignettes for full references.
}
\section{Output shape}{

Propagation helpers (\code{acs_sum}, \code{acs_diff}, \code{acs_ratio}, \code{acs_prop},
\code{acs_product}, \code{acs_linear}) return a data frame with columns \code{estimate},
\code{moe}, and \code{se}. \code{moe_ci()} returns \code{estimate}, \code{lower}, \code{upper}, \code{moe}.
\code{acs_simulate()} returns a numeric matrix of draws. \code{acs_simulate_fn()}
returns either \verb{(estimate, se)} for \code{summary = "mean"}/\code{"median"}, or
\verb{(estimate, lower, upper)} for \code{summary = "ci"}. \code{acs_cv()} and
\code{acs_reliability()} return atomic vectors.
}

\seealso{
Useful links:
\itemize{
  \item \url{https://dshkol.github.io/acsmoe/}
  \item \url{https://github.com/dshkol/acsmoe}
  \item Report bugs at \url{https://github.com/dshkol/acsmoe/issues}
}

}
\author{
\strong{Maintainer}: Dmitry Shkolnik \email{dmitry@dshkol.com}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core.R
\name{acs_diff}
\alias{acs_diff}
\title{Propagate ACS uncertainty for a difference.}
\usage{
acs_diff(estimate1, moe1, estimate2, moe2, cov = 0, conf = 0.9)
}
\arguments{
\item{estimate1}{First estimate.}

\item{moe1}{MOE for \code{estimate1}.}

\item{estimate2}{Second estimate.}

\item{moe2}{MOE for \code{estimate2}.}

\item{cov}{Covariance between the two estimates on the standard-error scale.}

\item{conf}{Confidence level associated with input and output MOEs.}
}
\value{
A data frame with \code{estimate}, \code{moe}, and \code{se}.
}
\description{
Propagate ACS uncertainty for a difference.
}
\examples{
acs_diff(100, 10, 40, 8)
acs_diff(100, 10, 40, 8, cov = 5)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{acs_cv}
\alias{acs_cv}
\title{Calculate coefficient of variation from an ACS estimate and MOE.}
\usage{
acs_cv(estimate, moe, conf = 0.9)
}
\arguments{
\item{estimate}{Numeric estimate.}

\item{moe}{Numeric MOE.}

\item{conf}{Confidence level associated with \code{moe}.}
}
\value{
Numeric coefficient of variation, using standard error divided by
absolute estimate.
}
\description{
Calculate coefficient of variation from an ACS estimate and MOE.
}
\examples{
acs_cv(estimate = 1000, moe = 80)
acs_cv(estimate = c(1000, 100, 0), moe = c(80, 60, 5))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conversions.R
\name{moe_ci}
\alias{moe_ci}
\title{Convert an ACS MOE to lower and upper confidence bounds.}
\usage{
moe_ci(estimate, moe, conf_in = 0.9, conf_out = 0.95)
}
\arguments{
\item{estimate}{Numeric estimate.}

\item{moe}{Numeric margin of error.}

\item{conf_in}{Confidence level associated with \code{moe}.}

\item{conf_out}{Desired confidence level for the returned interval.}
}
\value{
A data frame with \code{estimate}, \code{lower}, \code{upper}, and \code{moe}.
}
\description{
Convert an ACS MOE to lower and upper confidence bounds.
}
\examples{
moe_ci(estimate = 100, moe = 10)
moe_ci(estimate = c(100, 200), moe = c(10, 25), conf_out = 0.99)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{acs_reliability}
\alias{acs_reliability}
\title{Categorize ACS estimate reliability by CV thresholds.}
\usage{
acs_reliability(
  estimate,
  moe,
  conf = 0.9,
  thresholds = c(reliable = 0.12, caveat = 0.4)
)
}
\arguments{
\item{estimate}{Numeric estimate.}

\item{moe}{Numeric MOE.}

\item{conf}{Confidence level associated with \code{moe}.}

\item{thresholds}{Named numeric vector with \code{reliable} and \code{caveat} CV
thresholds. Defaults of 0.12 and 0.40 reflect commonly used applied
conventions (e.g., reliable below 12\% CV, unreliable above 40\% CV); these
are not a Census Bureau standard and should be adjusted to your domain,
agency guidance, or project-specific quality standard.}
}
\value{
Ordered factor with levels \code{reliable}, \code{caveat}, \code{unreliable}.
}
\description{
Categorize ACS estimate reliability by CV thresholds.
}
\examples{
acs_reliability(estimate = 1000, moe = 80)
acs_reliability(estimate = c(1000, 200, 50), moe = c(80, 60, 50))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aggregate.R
\name{acs_aggregate}
\alias{acs_aggregate}
\title{Aggregate paired ACS estimate and MOE columns by group.}
\usage{
acs_aggregate(
  data,
  group_var,
  value_cols,
  moe_cols,
  cov_strategy = c("zero", "supplied", "constant"),
  cov_value = 0,
  conf = 0.9
)
}
\arguments{
\item{data}{A data frame containing ACS estimate and MOE columns.}

\item{group_var}{Name of the grouping column, supplied as a single string.}

\item{value_cols}{Character vector of estimate column names to aggregate.}

\item{moe_cols}{Character vector of MOE column names paired with
\code{value_cols}.}

\item{cov_strategy}{Covariance strategy: \code{"zero"}, \code{"supplied"}, or
\code{"constant"}.}

\item{cov_value}{For \code{"constant"}, a scalar correlation. For \code{"supplied"},
a named list of full covariance matrices keyed by estimate column.}

\item{conf}{Confidence level associated with input and output MOEs.}
}
\value{
A data frame with one row per group and aggregated estimate/MOE
columns.
}
\description{
Aggregate paired ACS estimate and MOE columns by group.
}
\details{
\code{cov_strategy = "constant"} interprets \code{cov_value} as a correlation,
not a covariance. This differs from scalar \code{cov} arguments in core
propagation functions, where a scalar means an off-diagonal covariance on
the standard-error scale.

Output rows are ordered by first appearance of each group level in \code{data},
not alphabetically.
}
\examples{
tracts <- data.frame(
  region = c("north", "north", "south", "south"),
  pop = c(1000, 1200, 900, 1100),
  pop_moe = c(120, 140, 100, 130)
)
acs_aggregate(tracts, "region", "pop", "pop_moe")
acs_aggregate(tracts, "region", "pop", "pop_moe",
              cov_strategy = "constant", cov_value = 0.25)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conversions.R
\name{se_to_moe}
\alias{se_to_moe}
\title{Convert a standard error to an ACS-style margin of error.}
\usage{
se_to_moe(se, conf = 0.9)
}
\arguments{
\item{se}{Numeric standard error.}

\item{conf}{Desired confidence level.}
}
\value{
Numeric margin of error.
}
\description{
Convert a standard error to an ACS-style margin of error.
}
\examples{
se_to_moe(c(6, 15, 60))
se_to_moe(6, conf = 0.95)
}
